import pandas as pd
from odhpy import utils
from pandas.api.types import is_datetime64_any_dtype as is_datetime


def set_index_dt(df: pd.DataFrame, dt_values=None, start_dt=None, cancel_if_index_are_datetimes=True, **kwargs):
    """
    Returns a dataframe with datetimes as the index.     
    If no optional arguments are provided, the function will look for a column named "Date" (not 
    case-sensitive) within the input dataframe. Otherwise dt_values or start_dt (assumes daily)
    may be provided. 

    Args:
        df (pd.DataFrame): _description_
        dt_values (_type_, optional): _description_. Defaults to None.
        start_dt (_type_, optional): _description_. Defaults to None.
        cancel_if_index_are_datetimes (bool, optional): _description_. Defaults to True.
    """
    if cancel_if_index_are_datetimes and is_datetime(df.index):
        return df
    
    if start_dt != None:
        df["Date"] = utils.get_dates(start_dt, days=len(df))    
    elif dt_values != None:
        nn = len(df)
        if len(dt_values) < nn:
            raise Exception("dt_values is shorter than the dataframe.") 
        df["Date"] = dt_values[:nn]
 
    col = [c for c in df.columns if c.upper().strip() == "DATE"]
    if len(col) > 0:
        df["Date"] = pd.to_datetime(df[col[0]], **kwargs)
        answer = df.set_index("Date")
        return answer
    else:
        raise Exception("Could not find 'Date' column.")



import pandas as pd
from odhpy import utils

def read_ts_csv(filename, date_format=r"%d/%m/%Y", **kwargs):
    """Reads a daily timeseries csv into a DataFrame, and sets the index to the Date.
    Assumed there is a column named "Date"

    Args:
        filename (_type_): _description_
        date_format (str, optional): defaults to "%d/%m/%Y" as per Fors. Other common formats include "%Y-%m-%d", "%Y/%m/%d".

    Returns:
        _type_: _description_
    """
    df = pd.read_csv(filename, **kwargs)
    df = utils.set_index_dt(df, format=date_format)
    return df


from datetime import datetime, timedelta


def get_wy(dates, wy_month=7, using_end_year=True):
    """
    Returns water years, as a list of ints, for a given array of dates. Use this to
    add water year info into a pandas DataFrame. 

    The default (using_end_year==True) follows the convention used for fiscal years, 
    whereby water years are labelled based on their end dates. E.g. the 2022 water 
    year is from 2021-07-01 to 2022-06-30 inclusive. This has the advantage that 
    results from the period, e.g. annual totals, are determined in the calendar year 
    corresponding to the water year. The alternative (using_end_year==False) will 
    report based on the start of the period.
    """
    if using_end_year:
        answer = [d.year if d.month < wy_month else d.year + 1 for d in dates]
    else:
        answer = [d.year + 1 if d.month < wy_month else d.year for d in dates]        
    return answer


def get_dates(start_date, end_date=None, days=0, years=1, include_end_date=False):
    """
    Generates a list of daily datetime values from a given start date. The length 
    may be defined by an end_date, or a number of days, or a number of years. This 
    function may be useful for working with daily datasets and models.
    """
    if (days > 0):
        # great, we already have the number of days
        pass
    elif (end_date != None):
        # use end_date
        days = (end_date - start_date).days
        days = days + 1 if include_end_date else days
    else:
        # use years
        end_date = datetime(start_date.year + years, start_date.month, start_date.day,
            start_date.hour, start_date.minute, start_date.second, start_date.microsecond)
        days = (end_date - start_date).days
    date_list = [start_date + timedelta(days=x) for x in range(days)]
    return date_list